#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:copyright:
    Wenjie Lei (lei@princeton.edu), 2016
:license:
    GNU General Public License, Version 3
    (http://www.gnu.org/copyleft/gpl.html)
"""

from __future__ import (absolute_import, division, print_function)

from .window import window_on_stream, window_on_trace  # NOQA
